@@ -20,18 +20,19 @@ module XMonad.Layout.IndependentScreens (
     workspaces',
     withScreens, onCurrentScreen,
     marshallPP,
+    whenCurrentOn,
     countScreens,
     -- * Converting between virtual and physical workspaces
     -- $converting
     marshall, unmarshall, unmarshallS, unmarshallW,
-    marshallWindowSpace, unmarshallWindowSpace
+    marshallWindowSpace, unmarshallWindowSpace, marshallSort
 ) where
 
 -- for the screen stuff
 import Control.Applicative((<*), liftA2)
 import Control.Arrow hiding ((|||))
 import Control.Monad
-import Data.List
+import Data.List (nub, genericLength)
 import Graphics.X11.Xinerama
 import XMonad
 import XMonad.StackSet hiding (filter, workspaces)
@@ -44,7 +45,7 @@ import XMonad.Hooks.DynamicLog
 --
 -- You can define your workspaces by calling @withScreens@:
 --
--- > myConfig = defaultConfig { workspaces = withScreens 2 ["web", "email", "irc"] }
+-- > myConfig = def { workspaces = withScreens 2 ["web", "email", "irc"] }
 --
 -- This will create \"physical\" workspaces with distinct internal names for
 -- each (screen, virtual workspace) pair.
@@ -72,6 +73,9 @@ import XMonad.Hooks.DynamicLog
 -- of virtual workspaces to functions of physical workspaces, which work
 -- by marshalling the virtual workspace name and the currently focused
 -- screen into a physical workspace name.
+--
+-- A complete example abusing many of the functions below is available in the
+-- "XMonad.Config.Dmwit" module.
 
 type VirtualWorkspace  = WorkspaceId
 type PhysicalWorkspace = WorkspaceId
@@ -110,9 +114,9 @@ onCurrentScreen f vws = screen . current >>= f . flip marshall vws
 --
 -- > main = do
 -- >   nScreens <- countScreens
--- >   xmonad $ defaultConfig {
+-- >   xmonad $ def {
 -- >     ...
--- >     workspaces = withScreens nScreens (workspaces defaultConfig),
+-- >     workspaces = withScreens nScreens (workspaces def),
 -- >     ...
 -- >     }
 --
@@ -140,6 +144,42 @@ marshallPP s pp = pp {
     ppSort              = fmap (marshallSort s) (ppSort pp)
     }
 
+-- | Take a pretty-printer and turn it into one that only runs when the current
+-- workspace is one associated with the given screen. The way this works is a
+-- bit hacky, so beware: the 'ppOutput' field of the input will not be invoked
+-- if either of the following conditions is met:
+--
+-- 1. The 'ppSort' of the input returns an empty list (when not given one).
+--
+-- 2. The 'ppOrder' of the input returns the exact string @\"\\0\"@.
+--
+-- For example, you can use this to create a pipe which tracks the title of the
+-- window currently focused on a given screen (even if the screen is not
+-- current) by doing something like this:
+--
+-- > ppFocus s = whenCurrentOn s def
+-- >     { ppOrder  = \(_:_:title:_) -> [title]
+-- >     , ppOutput = appendFile ("focus" ++ show s) . (++ "\n")
+-- >     }
+--
+-- Sequence a few of these pretty-printers to get a log hook that keeps each
+-- screen's title up-to-date.
+whenCurrentOn :: ScreenId -> PP -> PP
+whenCurrentOn s pp = pp
+    { ppSort = do
+        sort <- ppSort pp
+        return $ \xs -> case xs of
+            x:_ | unmarshallS (tag x) == s -> sort xs
+            _ -> []
+    , ppOrder = \i@(wss:_) -> case wss of
+        "" -> ["\0"] -- we got passed no workspaces; this is the signal from ppSort that this is a boring case
+        _  -> ppOrder pp i
+    , ppOutput = \out -> case out of
+        "\0" -> return () -- we got passed the signal from ppOrder that this is a boring case
+        _ -> ppOutput pp out
+    }
+
+-- | If @vSort@ is a function that sorts 'WindowSpace's with virtual names, then @marshallSort s vSort@ is a function which sorts 'WindowSpace's with physical names in an analogous way -- but keeps only the spaces on screen @s@.
 marshallSort :: ScreenId -> ([WindowSpace] -> [WindowSpace]) -> ([WindowSpace] -> [WindowSpace])
 marshallSort s vSort = pScreens . vSort . vScreens where
     onScreen ws = unmarshallS (tag ws) == s
