@@ -12,6 +12,9 @@
 --
 -- A wrapper for keybinding configuration that can list the available
 -- keybindings.
+--
+-- Note that xmonad>=0.11 has by default a list of the default keybindings
+-- bound to @M-S-/@ or @M-?@.
 --------------------------------------------------------------------
 
 module XMonad.Util.NamedActions (
@@ -21,6 +24,7 @@ module XMonad.Util.NamedActions (
     spawn',
     submapName,
     addDescrKeys,
+    addDescrKeys',
     xMessage,
 
     showKmSimple,
@@ -63,7 +67,7 @@ import qualified XMonad.StackSet as W
 -- > import XMonad.Util.EZConfig
 -- >
 -- > main = xmonad $ addDescrKeys ((mod4Mask, xK_F1), xMessage) myKeys
--- >                    defaultConfig { modMask = mod4Mask }
+-- >                    def { modMask = mod4Mask }
 -- >
 -- > myKeys c = (subtitle "Custom Keys":) $ mkNamedKeymap c $
 -- >    [("M-x a", addName "useless message" $ spawn "xmessage foo"),
@@ -187,7 +191,7 @@ smartSpace [] = []
 smartSpace xs = ' ':xs
 
 _test :: String
-_test = unlines $ showKm $ defaultKeysDescr XMonad.defaultConfig { XMonad.layoutHook = XMonad.Layout $ XMonad.layoutHook XMonad.defaultConfig }
+_test = unlines $ showKm $ defaultKeysDescr XMonad.def { XMonad.layoutHook = XMonad.Layout $ XMonad.layoutHook XMonad.def }
 
 showKm :: [((KeyMask, KeySym), NamedAction)] -> [String]
 showKm keybindings = padding $ do
@@ -223,7 +227,7 @@ addDescrKeys' (k,f) ks conf =
         keylist l = M.map getAction $ M.fromList $ ks l ^++^ [(k, shk l)]
     in conf { keys = keylist }
 
--- | A version of the default keys from 'XMonad.Config.defaultConfig', but with
+-- | A version of the default keys from the default configuration, but with
 -- 'NamedAction'  instead of @X ()@
 defaultKeysDescr :: XConfig Layout -> [((KeyMask, KeySym), NamedAction)]
 defaultKeysDescr conf@(XConfig {XMonad.modMask = modm}) =
